<?php
/*
Plugin Name: Ktai Entry
Plugin URI: http://wppluginsj.sourceforge.jp/ktai_entry/
Description: Make posts from messages sent by mobile phones.
Author: IKEDA Yuriko
Version: 0.7.0
Author URI: http://www.yuriko.net/cat/wordpress/
*/

/*  Copyright (c) 2008 yuriko

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

//define('KE_DEBUG', TRUE);
define('KE_LOGFILE', 'logs/error.log');

if (! defined('PLUGINDIR')) {
	define('PLUGINDIR', 'wp-content/plugins');
}

// ----- Prevent launch of wp-mail.php
if (preg_match('/^([^?]*)/', $_SERVER['REQUEST_URI'], $path) && basename($path[1], '.php') == 'wp-mail') {
	exit();
}

$Ktai = new Ktai_Entry();
if (! defined('WP_USE_THEMES')) {
	$KE_Prefs = new Ktai_Entry_PrefPane();
	add_action('admin_menu',  array($KE_Prefs, 'add_page'));
}

/* ==================================================
 *   Ktai_Entry class
   ================================================== */

class Ktai_Entry {
	private $plugin_dir;
	private $nonce = -1;

public function __construct() {
	$this->set_plugin_dir();
	load_plugin_textdomain('ktai_entry', $this->plugin_dir);
	if ($this->get_option('ke_retrieve_interval') > 0) {
		add_action('init', array($this , 'check_messages'));
	}
}

/* ==================================================
 * @param	none
 * @return	none
 */
private function set_plugin_dir() {
	$this->plugin_dir = preg_replace('!^.*' . PLUGINDIR . '!' , PLUGINDIR, dirname(__FILE__)) . '/';
	if (DIRECTORY_SEPARATOR == '\\') {
		$this->plugin_dir = str_replace('\\', '/', $this->plugin_dir);
	}
	if (! preg_match('!^' . PLUGINDIR . '!', $this->plugin_dir)) {
		$this->plugin_dir = PLUGINDIR . '/ktai_entry/';
	}
}

/* ==================================================
 * @param	string  $name
 * @return	mix     $value
 */
public function get_option($name, $return_default = FALSE) {
	if (! $return_default) {
		$value = get_option($name);
		if (FALSE !== $value) {
			return $value;
		}
	}
	// default values 
	switch ($name) {
	case 'ke_retrieve_interval':
		return 15;
	case 'ke_thumb_size':
		if (function_exists('wp_get_attachment_link')) {
			return 'thumbnail';
		} else {
			return '160';
		}
	default:
		return NULL;
	}
}

// ==================================================
public function get_wp_version() {
	$version = get_bloginfo('version');
	if (! is_numeric($version)) {
		$version = preg_replace('/[^.0-9]/', '', $version);
	}
	return $version;
}

/* ==================================================
 * @param	none
 * @return	none
 */
public function check_messages() {
	$last_checked = get_option('ke_last_checked');
	$interval     = $this->get_option('ke_retrieve_interval');
	if ($last_checked < 0 || $interval < 1 || $interval * 60 > (time() - $last_checked)) {
		return FALSE;
	}

	$errno    = '';
	$errstr   = '';
	$ping_url = $this->retrieve_url();
	$parts    = parse_url($ping_url);
	$argyle = @ fsockopen(
		$parts['host'],
		$_SERVER['SERVER_PORT'],
		$errno,
		$errstr,
		0.01);
	if ($argyle) {
		fputs($argyle, "GET {$parts['path']}?{$parts['query']} HTTP/1.0\r\nHost: {$_SERVER['HTTP_HOST']}\r\n\r\n");
		update_option('ke_last_checked', time());
	}
	return TRUE;
}


/* ==================================================
 * @param	none
 * @return	string   $url
 */
public function retrieve_url() {
	$i = ceil(time() / 43200);
	$nonce = substr(wp_hash($i . 'ktai-entry-retrive' . $_SERVER['SERVER_NAME']), -12, 10) ;
	$url = get_bloginfo('wpurl') . '/' . $this->plugin_dir . 'retrieve.php?_wpnonce=' . $nonce;
	return $url;
}

/* ==================================================
 * @param	string  $message
 * @return	none
 */
public function debug_print($message) {
	if (defined('KE_DEBUG') && defined('KE_LOGFILE')) {
		$fh = @ fopen(dirname(__FILE__) . '/' . KE_LOGFILE, 'a');
		if ($fh) {
			fwrite($fh, date("Y-m-d H:i:s ") . $message . "\n");
			fclose($fh);
		}
	}
}

// ===== End of class ====================
}

/* ==================================================
 *   Ktai_Entry_PrefPane class
   ================================================== */

class Ktai_Entry_PrefPane extends Ktai_Entry {
	var $nonce = -1;

/* ==================================================
 * @param	none
 * @return	none
 */
public function add_page() {
	add_options_page('Ktai Entry Configuration', __('Email Post', 'ktai_entry'), 'manage_options', basename(__FILE__), array($this, 'option_page'));
	if ( !function_exists('wp_nonce_field') ) {
		$this->nonce = -1;
	} else {
		$this->nonce = 'ktai-entry-config';
	}
}

/* ==================================================
 * @param	none
 * @return	none
 */
public function option_page() {
	global $user_identity;

	if (isset($_POST['update_option'])) {
		check_admin_referer($this->nonce);
		$this->upate_options();
	}
	$posting_addr      = $this->get_option('ke_posting_addr');
	$use_apop          = $this->get_option('ke_use_apop');
	$retrieve_interval = intval($this->get_option('ke_retrieve_interval'));
	$retrive_selection = array(
		0  => __('Never', 'ktai_entry'),
		5  => __('5 min', 'ktai_entry'),
		10 => __('10 min', 'ktai_entry'),
		15 => __('15 min', 'ktai_entry'),
		20 => __('20 min', 'ktai_entry'),
		30 => __('30 min', 'ktai_entry'),
		60 => __('1 hour', 'ktai_entry'),
		120 => __('2 hour', 'ktai_entry'),
		);
	$thumb_max_size = $this->get_option('ke_thumb_size');
	if ($thumb_max_size == 'medium') {
		$thumb_size[0] = '';
		$thumb_size[1] = ' checked="checked"';
	} else {
		$thumb_size[0] = ' checked="checked"';
		$thumb_size[1] = '';
	}
?>
<div class="wrap">
<h2><?php _e('Ktai Entry Options', 'ktai_entry'); ?></h2>
<p><?php _e('Note: To configure POP3 mail server, go <a href="options-writing.php">Writing Options</a>.', 'ktai_entry'); ?></p>
<form method="post">
<?php $this->make_nonce_field($this->nonce); ?>
<table class="optiontable form-table"><tbody>
<tr>
<th><label for="posting_addr"><?php _e('Posting mail address', 'ktai_entry'); ?></label></th>
<td><input type="text" value="<?php echo attribute_escape($posting_addr); ?>" name="posting_addr" id="posting_addr" size="64" /><br />
<small><?php _e('Reject all mail whose recipients are not this address.', 'ktai_entry');  ?></small></td>
</tr><tr>
<th>APOP</th>
<td><label><input type="checkbox"<?php if ($use_apop) {echo ' checked="checked"';} ?>name="use_apop" id="use_apop" /> <?php _e('Use APOP', 'ktai_entry');  ?></label></td>
</tr><tr>
<th><label for="retrieve_interval"><?php _e('POP3 retrieve interval', 'ktai_entry'); ?></label></th>
<td><select name="retrieve_interval" id="retrieve_interval">
<?php
	$selected = FALSE;
	foreach ($retrive_selection as $m => $n) {
		if (intval($m) == $retrieve_interval) {
			$sel_html = ' selected="selected"';
			$selected = TRUE;
		} else {
			$sel_html = '';
		}
		echo '<option value="' . intval($m) . '"' . $sel_html . '>' . $n . "</option>\n";
	}
	if (! $selected) {
		echo '<option value="' . $retrieve_interval . '" selected="selected">' . $retrieve_interval . __('min', 'ktai_entry') . "</option>\n";
	}
?>
</select><br /><small><?php 
	$url = $this->retrieve_url();
	printf(__('Retrieve messages now if you fetch <a href="%s">%s</a>.', 'ktai_entry'), $url, $url); ?></small></td>
</tr><tr>
<?php if (function_exists('wp_get_attachment_link')) { ?>
<th><label for="thumb_size"><?php _e('Image size of inserting into post', 'ktai_entry'); ?></label></th>
<td>
	<label><input type="radio" name="thumb_size" value="thumbnail"<?php echo $thumb_size[0]; ?> /> <?php _e('Thumbnail'); ?></label><br />
	<label><input type="radio" name="thumb_size" value="medium"<?php echo $thumb_size[1]; ?> /> <?php _e('Medium'); ?></label>
</td>
<?php } else { ?>
<th><label for="thumb_size"><?php _e('Max size of the image thumbnail', 'ktai_entry'); ?></label></th>
<td><input type="text" value="<?php echo intval($thumb_max_size); ?>" name="thumb_size" id="thumb_size" /></td>
<?php } ?>
</tr>
</tbody></table>
<div class="submit">
<input type="hidden" name="action" value="update" />
<input type="submit" name="update_option" value="<?php echo (version_compare($this->get_wp_version(), '2.5', '>=') ? __('Save Changes') : __('Update Options &raquo;')) ?>" />
</div>
</form>
</div>
<?php
} 

/* ==================================================
 * @param	mix   $action
 * @return	none
 */
private function make_nonce_field($action = -1) {
	if ( !function_exists('wp_nonce_field') ) {
		return;
	} else {
		return wp_nonce_field($action);
	}
}

/* ==================================================
 * @param	none
 * @return	none
 */
private function upate_options() {
	if (isset($_POST['posting_addr'])) {
		update_option('ke_posting_addr', $_POST['posting_addr']);
	} else {
		delete_option('ke_posting_addr');
	}

	if (isset($_POST['use_apop'])) {
		update_option('ke_use_apop', TRUE);
	} else {
		update_option('ke_use_apop', FALSE);
	}

	if (FALSE !== @$_POST['retrieve_interval']) {
		if (is_numeric($_POST['retrieve_interval']) && ($interval = intval($_POST['retrieve_interval'])) >= 0 ) {
			update_option('ke_retrieve_interval', $interval);
		}
	} else {
		delete_option('ke_retrieve_interval');
	}

	if (isset($_POST['thumb_size'])) {
		if (function_exists('wp_get_attachment_link')) {
			if ($_POST['thumb_size'] == 'thumbnail' || $_POST['thumb_size'] == 'medium') {
				update_option('ke_thumb_size', $_POST['thumb_size']);
			}
		} elseif (is_numeric($_POST['thumb_size']) && intval($_POST['thumb_size']) > 9) {
			update_option('ke_thumb_size', intval($_POST['thumb_size']));
		}
	} else {
		delete_option('ke_thumb_size');
	}
?>
<div class="updated fade"><p><strong><?php _e('Options saved.'); ?></strong></p></div>
<?php
	return;
}

// ===== End of class ====================
}
?>